--------------------------------------------------------------------------------
-- GLOBALS
--------------------------------------------------------------------------------

--      :
Global("HealthForm", nil)

--        (     ):
Global("LastEventAvatarWillBeRemovedDelay", 0)

--    :
Global("MobsDb", nil) --[[{
	--  :
	Index = {
		--  :
		Name = WString,
		--  :
		Level = number,
		--      :
		DamageSum = number,
		--      :
		PercentSum = number
	}
}]]

--   :
Global("Units", {}) --[[{
	--  :
	Id = {
		--       ,   nil:
		MobDb = MobsDb[...],
		--    :
		DamageDelta = number,
		--    :
		PercentBeg = number,
		--     :
		PercentLast = number,
		--       ,    :
		PlayerDamageSum = number,
		--       ,    :
		PlayerPercentSum = number
	}
}]]

--------------------------------------------------------------------------------
-- FUNCTIONS
--------------------------------------------------------------------------------

--     ( ,   ):
function GetHealthValue(iDamageDelta, iPercentDelta)
	if iPercentDelta == 0 or iDamageDelta == 0 then
		return 0
	else
		return iDamageDelta * 100 / iPercentDelta
	end
end

--      :
function UpdateForm()
	--    :
	if HealthForm:IsVisible() then
			HealthForm:Show(false)
	end
	--  :
	local vTargetId = avatar.GetTarget()
	-- :
	local aId =  avatar.GetId()
	--    , : 
	if vTargetId ~= aId then
		--  ,   :
		local vUnit = Units[vTargetId]
		--   :
		local vHealth = 0
		local vHealthMax = 0
		--          :
		if vUnit then
			local vDamageSum = vUnit.PlayerDamageSum
			local vPercentSum = vUnit.PlayerPercentSum
			--  :
			if vUnit.MobDb then
				vDamageSum = vUnit.MobDb.DamageSum
				vPercentSum = vUnit.MobDb.PercentSum
			end
			--       :
			if vUnit.DamageDelta > 0 then
				vDamageSum = vDamageSum + vUnit.DamageDelta
			else
				vDamageSum = vDamageSum - vUnit.DamageDelta
			end
			--        ()   ():
			local vPercentDelta = vUnit.PercentBeg - vUnit.PercentLast
			if vPercentDelta > 0 then
				vPercentSum = vPercentSum + vPercentDelta
			else
				vPercentSum = vPercentSum - vPercentDelta
			end
			--   :
			vHealthMax = GetHealthValue(vDamageSum, vPercentSum)
			--   :
			vHealth = unit.GetHealthPercentage(vTargetId) * vHealthMax / 100
		end
		--    :
		HealthForm:SetVal("CurrentHealth", common.FormatInt(math.ceil(vHealth), "%dK5"))
		HealthForm:SetVal("MaxHealth", common.FormatInt(math.ceil(vHealthMax), "%dK5"))
		--       :
		if vHealth > 0 then
			if not HealthForm:IsVisible() then
				HealthForm:Show(true)
			end
		end
	else
		--  ,    :
		if HealthForm:IsVisible() then
			HealthForm:Show(false)
		end
	end
end

--    ( ):
function CreateUnit(iUnitId)
	--  :
	local vUnit = {}
	--         :
	if not unit.IsPlayer(iUnitId) and not unit.IsPet(iUnitId) then
		local vName = object.GetName(iUnitId)
		local vLevel = unit.GetLevel(iUnitId)
		local vCount = 0
		for k, vMobDb in MobsDb do
			if common.CompareWString(vMobDb.Name, vName) == 0 and vMobDb.Level == vLevel then
				vUnit.MobDb = vMobDb
				break
			end
			vCount = vCount + 1
		end
		if not vUnit.MobDb then
			vUnit.MobDb = {}
			vUnit.MobDb.Name = vName
			vUnit.MobDb.Level = vLevel
			vUnit.MobDb.DamageSum = 0
			vUnit.MobDb.PercentSum = 0
			MobsDb[vCount] = vUnit.MobDb
		end
	end
	--   :
	vUnit.DamageDelta = 0
	vUnit.PercentBeg = unit.GetHealthPercentage(iUnitId)
	vUnit.PercentLast = vUnit.PercentBeg
	vUnit.PlayerDamageSum = 0
	vUnit.PlayerPercentSum = 0
	--    :
	Units[iUnitId] = vUnit
end

--       ( ):
function DestroyUnit(iUnitId)
	--  :
	local vUnit = Units[iUnitId]
	if vUnit then
		--   :
		if vUnit.MobDb then
			--       :
			local vPercentDelta = vUnit.PercentBeg - vUnit.PercentLast
			if vPercentDelta <= -10 or 10 <= vPercentDelta then
				local vDamageDelta = vUnit.DamageDelta
				if vPercentDelta < 0 then
					vDamageDelta = -vDamageDelta
					vPercentDelta = -vPercentDelta
				end
				--     :
				if vDamageDelta > 0 then
					--     :
					vUnit.MobDb.DamageSum = vUnit.MobDb.DamageSum + vDamageDelta
					vUnit.MobDb.PercentSum = vUnit.MobDb.PercentSum + vPercentDelta
				end
			end
		end
		--   :
		Units[iUnitId] = nil
	end
end

--------------------------------------------------------------------------------
-- EVENT HANDLERS
--------------------------------------------------------------------------------

--    :
function OnEVENT_AVATAR_SECONDARY_TARGET_CHANGED(param)
	--    ,    :
	local vTargetId = avatar.GetTarget()
	if vTargetId and not Units[vTargetId] then
		CreateUnit(vTargetId)
	end
	--  :
	UpdateForm()
end

--      :
function OnEVENT_UNIT_SPAWNED(param)
	--   :
	if not Units[param.unitId] then
		CreateUnit(param.unitId)
	end
end

--      :
function OnEVENT_UNIT_DESPAWNED(param)
	--      :
	if Units[param.unitId] then
		DestroyUnit(param.unitId)
	end
end

--    :
function OnEVENT_UNIT_DAMAGE_RECEIVED(param)
	--   :
	local vUnit = Units[param.target]
	if vUnit then
		--  ,  ,    ,    DamageDelta:
		local vPercent = unit.GetHealthPercentage(param.target)
		if 0 < vPercent then
			vUnit.DamageDelta = vUnit.DamageDelta + param.amount
			vUnit.PercentLast = vPercent
		else
			--  ,     :
			DestroyUnit(param.target)
		end
	else
		--   ,  :
		CreateUnit(param.target)
	end
	--  :
	if param.target == avatar.GetTarget() then
		UpdateForm()
	end
end

--    :
function OnEVENT_HEALING_RECEIVED(param)
	--   :
	local vUnit = Units[param.unitId]
	if vUnit then
		--   :
		local vPercent = unit.GetHealthPercentage(param.unitId)
		--        :
		if unit.IsPlayer(param.unitId) then
			local vPercentDelta = vUnit.PercentBeg - vUnit.PercentLast
			--     :
			if 10 <= vPercentDelta and 0 <= vUnit.DamageDelta then
				--     :
				vUnit.PlayerDamageSum = vUnit.PlayerDamageSum + vUnit.DamageDelta
				vUnit.PlayerPercentSum = vUnit.PlayerPercentSum + vPercentDelta
			end
			--         :
			vUnit.DamageDelta = 0
			vUnit.PercentBeg = vPercent
			vUnit.PercentLast = vPercent
		elseif vPercent < 100 then
			--  ,  ,    ,    DamageDelta:
			vUnit.DamageDelta = vUnit.DamageDelta - param.heal
			vUnit.PercentLast = vPercent
		else
			--   ,     :
			DestroyUnit(param.unitId)
		end
	else
		--   ,  :
		CreateUnit(param.unitId)
	end
	--  :
	if param.unitId == avatar.GetTarget() then
		UpdateForm()
	end
end

-- 
-- 
--
function OnEVENT_AVATAR_CLIENT_ZONE_CHANGED(param)
	--    :
	local vMobsDb = {}
	local vMobsDbLength = 0
	for k, vMobDb in MobsDb do
		if vMobDb.DamageSum > 0 then
			vMobsDb[vMobsDbLength] = vMobDb
			vMobsDbLength = vMobsDbLength + 1
		end
	end
	userMods.SetGlobalConfigSection("HealthDb", vMobsDb)
end

--     (   ):
function OnEVENT_AVATAR_WILL_BE_REMOVED(param)
	--         :
	if LastEventAvatarWillBeRemovedDelay <= param.delay then
		--    :
		local vMobsDb = {}
		local vMobsDbLength = 0
		for k, vMobDb in MobsDb do
			if vMobDb.DamageSum > 0 then
				vMobsDb[vMobsDbLength] = vMobDb
				vMobsDbLength = vMobsDbLength + 1
			end
		end
		userMods.SetGlobalConfigSection("HealthDb", vMobsDb)
	end
	--        :
	LastEventAvatarWillBeRemovedDelay = param.delay
end

--------------------------------------------------------------------------------
-- INITIALIZATION
--------------------------------------------------------------------------------
function Init()
	-- :
	common.RegisterEventHandler(OnEVENT_AVATAR_SECONDARY_TARGET_CHANGED, "EVENT_AVATAR_SECONDARY_TARGET_CHANGED")
	common.RegisterEventHandler(OnEVENT_UNIT_SPAWNED, "EVENT_UNIT_SPAWNED")
	common.RegisterEventHandler(OnEVENT_UNIT_DESPAWNED, "EVENT_UNIT_DESPAWNED")
	common.RegisterEventHandler(OnEVENT_UNIT_DAMAGE_RECEIVED, "EVENT_UNIT_DAMAGE_RECEIVED")
	common.RegisterEventHandler(OnEVENT_HEALING_RECEIVED, "EVENT_HEALING_RECEIVED")
	common.RegisterEventHandler(OnEVENT_AVATAR_WILL_BE_REMOVED, "EVENT_AVATAR_WILL_BE_REMOVED")
	common.RegisterEventHandler(OnEVENT_AVATAR_CLIENT_ZONE_CHANGED, "EVENT_AVATAR_CLIENT_ZONE_CHANGED")
	--      :
	HealthForm = mainForm:GetChildChecked("HealthForm", false)
	--    :
	MobsDb = userMods.GetGlobalConfigSection("HealthDb")
	if not MobsDb then
		MobsDb = {}
	end
end
--------------------------------------------------------------------------------
Init()
--------------------------------------------------------------------------------